import json
import os
import subprocess
import sys
from typing import Any, Dict, List, Optional, Union

import toml
from beaupy import select_multiple
from pyfiglet import Figlet
from rich.console import Console
from rich.panel import Panel
from termcolor import cprint


def install_playwright_browsers() -> None:
    console: Console = Console()
    try:
        from playwright.sync_api import Browser, sync_playwright

        with sync_playwright() as p:
            browser: Browser = p.chromium.launch()
            browser.close()
    except Exception as e:
        print(f"Error: {e}")
        print("Installing Playwright browsers...")
        subprocess.run([sys.executable, "-m", "playwright", "install"], check=True)
        print("Playwright browsers installed successfully.")
    console.clear()


def get_version_from_toml() -> str:
    try:
        pyproject: Dict[str, Any] = toml.load("pyproject.toml")
        return pyproject.get("tool", {}).get("poetry", {}).get("version", "unknown")
    except Exception as e:
        print(f"Error reading version from pyproject.toml: {e}")
        return "unknown"


def show_banner() -> None:
    try:
        width: int = os.get_terminal_size()[0]
    except Exception:
        width = 80

    banner: Figlet = Figlet(font="big", justify="center", width=width)
    cprint(banner.renderText("FollowSphere"), color="blue")

    version: str = get_version_from_toml()
    url: str = "https://hyperoot.dev"
    console: Console = Console()
    console.print(Panel(f"{url} - v{version}"), justify="center", style="green")


def read_data() -> Optional[Dict]:
    try:
        script_dir: str = os.path.dirname(os.path.abspath(__file__))
        data_file_path: str = os.path.join(script_dir, "data.json")
        with open(data_file_path, "r") as file:
            data: Dict = json.load(file)
        return data
    except FileNotFoundError:
        print("File not found")
        exit()


def tags_follower(data: Dict[str, Any]) -> Optional[List[str]]:
    console: Console = Console()
    console.clear()
    show_banner()
    options: List[str] = []
    for k, _v in data.items():
        options.append(k)

    console.print("Which tag collection do you want to follow?\n", style="Cyan")
    items: List[Union[int, Any]] = select_multiple(
        sorted(options), tick_character="✓", ticked_indices=None, tick_style="green"
    )
    if items:
        console.print(f"\nSelected tag collections: [green]{" ".join(items)}[/green]")
        tags: List[str] = []
        for item in items:
            if data[item]["tags"]:
                for tag in data[item]["tags"]:
                    if tag not in tags:
                        tags.append(tag)
        return tags
    else:
        console.print("You didn't select anything. Terminating the program.")
        exit()



import asyncio
from typing import Any, Dict, List

from beaupy import select
from playwright.async_api import Browser, ElementHandle, Page, async_playwright
from rich.console import Console

from followsphere.utils import read_data, show_banner, tags_follower


async def login(page: Page) -> None:
    await page.goto("https://www.linkedin.com/login")
    await page.wait_for_selector(".authentication-outlet")


async def follow_hashtags(page: Page, hashtags: List[str]) -> None:
    for hashtag in hashtags:
        try:
            await page.goto(f"https://www.linkedin.com/feed/hashtag/{hashtag.lower()}/")
            follow_button: ElementHandle = await page.query_selector("button.follow")
            if follow_button:
                # Check if the button has the 'is-following' class
                class_name: str = await follow_button.get_attribute("class")
                if "is-following" in class_name:
                    print(f"Already following #{hashtag}")
                else:
                    # Click the follow button if it's not followed yet
                    await follow_button.click()
                    print(f"Clicked follow for [green]#{hashtag}[/green]")
        except Exception as e:
            print(f"An error occurred for #{hashtag}: {e}")


async def entrypoint(hashtags: List[str]) -> None:
    async with async_playwright() as p:
        browser: Browser = await p.chromium.launch(headless=False)  # Set to False to see the browser in action
        page: Page = await browser.new_page()
        await login(page)
        await follow_hashtags(page, hashtags)
        await browser.close()


def execute_linkedin() -> None:
    console: Console = Console()
    console.clear()
    show_banner()
    data: Dict[str, Any] = read_data()["LinkedIn"]
    options: List = []
    for k, _v in data.items():
        options.append(k)
    console.print("Choose your collection (Use arrow keys)\n", style="Cyan")
    option: str = select(sorted(options), cursor="\uf061", cursor_style="red")
    if option == "Role based Collections":
        hashtags: List[str] = tags_follower(data["Role based Collections"])
        asyncio.run(entrypoint(hashtags))
    elif option == "Skill based Collections":
        hashtags: List[str] = tags_follower(data["Skill based Collections"])
        asyncio.run(entrypoint(hashtags))
    else:
        print("\nInvalid option selected")



from typing import List

import typer
from beaupy import select
from rich.console import Console

from followsphere.linkedin import execute_linkedin
from followsphere.utils import install_playwright_browsers, show_banner


def main():
    console: Console = Console()
    console.clear()
    install_playwright_browsers()
    show_banner()
    options: List[str] = [
        "Linkedin",
        "Instagram (Coming soon)",
    ]
    console.print("Pick a platform (Use arrow keys)\n", style="Cyan")
    option: str = select(sorted(options), cursor="\uf061", cursor_style="red")
    if option.lower() == "linkedin":
        typer.run(execute_linkedin)
    else:
        typer.echo("\nInvalid platform selected.")


if __name__ == "__main__":
    typer.run(main)
